@@ -14,7 +14,6 @@ backwards compatibility.
 
 import io
 import logging
-import tempfile
 import os
 import sys
 import hashlib
@@ -25,11 +24,12 @@ import mock
 from moto import mock_s3
 import responses
 import gzip
-import six
 
 import smart_open
 from smart_open import smart_open_lib
 
+from .test_smart_open import named_temporary_file
+
 logger = logging.getLogger(__name__)
 
 PY2 = sys.version_info[0] == 2
@@ -70,7 +70,6 @@ class SmartOpenHttpTest(unittest.TestCase):
         self.assertTrue(actual_request.headers['Authorization'].startswith('Basic '))
 
     @responses.activate
-    @unittest.skipIf(six.PY2, 'gzip support for Py2 is not implemented yet')
     def test_http_gz(self):
         """Can open gzip via http?"""
         fpath = os.path.join(CURR_DIR, 'test_data/crlf_at_1k_boundary.warc.gz')
@@ -88,7 +87,6 @@ class SmartOpenHttpTest(unittest.TestCase):
         self.assertEqual(m.hexdigest(), expected_hash)
 
     @responses.activate
-    @unittest.skipIf(six.PY2, 'gzip support for Py2 is not implemented yet')
     def test_http_gz_noquerystring(self):
         """Can open gzip via http?"""
         fpath = os.path.join(CURR_DIR, 'test_data/crlf_at_1k_boundary.warc.gz')
@@ -113,7 +111,7 @@ class SmartOpenHttpTest(unittest.TestCase):
         # TODO: why are these tests writing to temporary files?  We can do the
         # bz2 compression in memory.
         #
-        with tempfile.NamedTemporaryFile('wb', suffix='.bz2', delete=False) as infile:
+        with named_temporary_file('wb', suffix='.bz2', delete=False) as infile:
             test_file = infile.name
 
         with smart_open.smart_open(test_file, 'wb') as outfile:
@@ -171,9 +169,6 @@ class SmartOpenReadTest(unittest.TestCase):
             actual = fin.read()
         self.assertEqual(expected, actual)
 
-    @unittest.skipUnless(
-        smart_open_lib.PATHLIB_SUPPORT,
-        "do not test pathlib support if pathlib or backport are not available")
     def test_open_and_read_pathlib_path(self):
         """If ``pathlib.Path`` is available we should be able to open and read."""
         from smart_open.smart_open_lib import pathlib
@@ -286,7 +281,7 @@ class SmartOpenReadTest(unittest.TestCase):
         short_path = "~/tmp/test.txt"
         full_path = os.path.expanduser(short_path)
 
-    @mock.patch(_IO_OPEN if six.PY2 else _BUILTIN_OPEN)
+    @mock.patch(_BUILTIN_OPEN)
     def test_file_errors(self, mock_smart_open):
         prefix = "file://"
         full_path = '/tmp/test.txt'
@@ -546,8 +541,7 @@ class SmartOpenTest(unittest.TestCase):
         #
 
     def test_text(self):
-        patch = _IO_OPEN if six.PY2 else _BUILTIN_OPEN
-        with mock.patch(patch, mock.Mock(return_value=self.stringio)) as mock_open:
+        with mock.patch(_BUILTIN_OPEN, mock.Mock(return_value=self.stringio)) as mock_open:
             with smart_open.smart_open("blah", "r", encoding='utf-8') as fin:
                 self.assertEqual(fin.read(), self.as_text)
                 mock_open.assert_called_with("blah", "r", buffering=-1, encoding='utf-8')
@@ -576,22 +570,19 @@ class SmartOpenTest(unittest.TestCase):
 
     def test_write_utf8(self):
         # correct write mode, correct file:// URI
-        patch = _IO_OPEN if six.PY2 else _BUILTIN_OPEN
-        with mock.patch(patch, mock.Mock(return_value=self.stringio)) as mock_open:
+        with mock.patch(_BUILTIN_OPEN, mock.Mock(return_value=self.stringio)) as mock_open:
             with smart_open.smart_open("blah", "w", encoding='utf-8') as fout:
                 mock_open.assert_called_with("blah", "w", buffering=-1, encoding='utf-8')
                 fout.write(self.as_text)
 
     def test_write_utf8_absolute_path(self):
-        patch = _IO_OPEN if six.PY2 else _BUILTIN_OPEN
-        with mock.patch(patch, mock.Mock(return_value=self.stringio)) as mock_open:
+        with mock.patch(_BUILTIN_OPEN, mock.Mock(return_value=self.stringio)) as mock_open:
             with smart_open.smart_open("/some/file.txt", "w", encoding='utf-8') as fout:
                 mock_open.assert_called_with("/some/file.txt", "w", buffering=-1, encoding='utf-8')
                 fout.write(self.as_text)
 
     def test_append_utf8(self):
-        patch = _IO_OPEN if six.PY2 else _BUILTIN_OPEN
-        with mock.patch(patch, mock.Mock(return_value=self.stringio)) as mock_open:
+        with mock.patch(_BUILTIN_OPEN, mock.Mock(return_value=self.stringio)) as mock_open:
             with smart_open.smart_open("/some/file.txt", "w+", encoding='utf-8') as fout:
                 mock_open.assert_called_with("/some/file.txt", "w+", buffering=-1, encoding='utf-8')
                 fout.write(self.as_text)
@@ -681,7 +672,7 @@ class SmartOpenTest(unittest.TestCase):
         text = u'欲しい気持ちが成長しすぎて'
 
         with self.assertRaises(UnicodeEncodeError):
-            with tempfile.NamedTemporaryFile('wb', delete=True) as infile:
+            with named_temporary_file('wb', delete=True) as infile:
                 with smart_open.smart_open(infile.name, 'w', encoding='koi8-r',
                                            errors='strict') as fout:
                     fout.write(text)
@@ -692,7 +683,7 @@ class SmartOpenTest(unittest.TestCase):
         text = u'欲しい気持ちが成長しすぎて'
         expected = u'?' * len(text)
 
-        with tempfile.NamedTemporaryFile('wb', delete=True) as infile:
+        with named_temporary_file('wb', delete=True) as infile:
             with smart_open.smart_open(infile.name, 'w', encoding='koi8-r',
                                        errors='replace') as fout:
                 fout.write(text)
@@ -771,10 +762,6 @@ class WebHdfsWriteTest(unittest.TestCase):
 
 @mock.patch('warnings.warn', mock.Mock())
 class CompressionFormatTest(unittest.TestCase):
-    """
-    Test that compression
-    """
-
     TEXT = 'Hello'
 
     def write_read_assertion(self, test_file):
@@ -784,8 +771,15 @@ class CompressionFormatTest(unittest.TestCase):
         with smart_open.smart_open(test_file, 'rb') as fin:
             self.assertEqual(fin.read().decode('utf8'), self.TEXT)
 
-        if os.path.isfile(test_file):
+        try:
             os.unlink(test_file)
+        except PermissionError:
+            #
+            # Work-around for Windows, see:
+            #
+            # <https://github.com/RaRe-Technologies/smart_open/issues/482>
+            #
+            pass
 
     def test_open_gz(self):
         """Can open gzip?"""
@@ -798,13 +792,13 @@ class CompressionFormatTest(unittest.TestCase):
 
     def test_write_read_gz(self):
         """Can write and read gzip?"""
-        with tempfile.NamedTemporaryFile('wb', suffix='.gz', delete=False) as infile:
+        with named_temporary_file('wb', suffix='.gz', delete=False) as infile:
             test_file_name = infile.name
         self.write_read_assertion(test_file_name)
 
     def test_write_read_bz2(self):
         """Can write and read bz2?"""
-        with tempfile.NamedTemporaryFile('wb', suffix='.bz2', delete=False) as infile:
+        with named_temporary_file('wb', suffix='.bz2', delete=False) as infile:
             test_file_name = infile.name
         self.write_read_assertion(test_file_name)
 
@@ -861,9 +855,8 @@ class MultistreamsBZ2Test(unittest.TestCase):
     )
 
     def create_temp_bz2(self, streams=1):
-        f = tempfile.NamedTemporaryFile('wb', suffix='.bz2', delete=False)
-        f.write(self.DATA * streams)
-        f.close()
+        with named_temporary_file('wb', suffix='.bz2', delete=False) as f:
+            f.write(self.DATA * streams)
         return f.name
 
     def cleanup_temp_bz2(self, test_file):
@@ -973,7 +966,6 @@ class S3OpenTest(unittest.TestCase):
         with smart_open.smart_open(key, "rb") as fin:
             self.assertEqual(fin.read().decode("utf-8"), text)
 
-    @unittest.skipIf(six.PY2, 'this test does not work with Py2')
     @mock_s3
     def test_gzip_write_mode(self):
         """Should always open in binary mode when writing through a codec."""
@@ -984,7 +976,6 @@ class S3OpenTest(unittest.TestCase):
             smart_open.smart_open("s3://bucket/key.gz", "wb")
             mock_open.assert_called_with('bucket', 'key.gz', 'wb')
 
-    @unittest.skipIf(six.PY2, 'this test does not work with Py2')
     @mock_s3
     def test_gzip_read_mode(self):
         """Should always open in binary mode when reading through a codec."""
